#include "stdafx.h"
#include "usbdrv.h"
#include "usb_descriptors.h"
#include "avr910.h"
#include <string.h>

/* USB CDC host command */
enum
{
    SEND_ENCAPSULATED_COMMAND = 0,
    GET_ENCAPSULATED_RESPONSE,
    SET_COMM_FEATURE,
    GET_COMM_FEATURE,
    CLEAR_COMM_FEATURE,
    SET_LINE_CODING = 0x20,
    GET_LINE_CODING,
    SET_CONTROL_LINE_STATE,
    SEND_BREAK
};

static UCHAR	intr3Status;    // used to control interrupt endpoint transmissions
static UCHAR	baudsetup[8];

/*****************************************************************************

******************************************************************************/
UCHAR usbFunctionSetup(UCHAR data[8])
{
	usbRequest_t *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS)
    {
      	// class request type
		if( rq->bRequest==GET_LINE_CODING || rq->bRequest==SET_LINE_CODING )
        {
            // GET_LINE_CODING -> usbFunctionRead()
        	// SET_LINE_CODING -> usbFunctionWrite()
           	return 0xff;
        }
        if(rq->bRequest == SET_CONTROL_LINE_STATE)
        {
            // Report serial state (carrier detect). On several Unix platforms,
            // tty devices can only be opened when carrier detect is set.
            intr3Status = 2;
        }
    }

    return 0;
}


/*****************************************************************************

******************************************************************************/
UCHAR usbFunctionRead( UCHAR *data, UCHAR len )
{
	memcpy(data, baudsetup, len);
	return 7;
}


/*****************************************************************************

******************************************************************************/
UCHAR usbFunctionWrite( UCHAR *data, UCHAR len )
{
	// SET_LINE_CODING
  	memcpy(baudsetup, data, len);
	return 1;
}

/*****************************************************************************

******************************************************************************/
void usbFunctionWriteOut(UCHAR *data, UCHAR len)
{	
  	usbDisableAllRequests();
    AVR910_Command(data, len);

// For each block we receive from the host, we schedule an empty block in the
// opposite direction. This fixed a hang on Windows. Since this is a hack,
// check from time to time whether it is still required!

#if 1

    if(usbInterruptIsReady())
    	usbSetInterrupt(0, 0);

#endif
}


/*****************************************************************************

******************************************************************************/
static void hardwareInit(void)
{
	/*  pull-ups   USB */
    USB_CFG_IOPORT &= ~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
	
    /*         */
    (*(ULONG *)(baudsetup[0])) = (ULONG)115200; /*   115200  */
    baudsetup[4] = 0; /*  stopbit */
	baudsetup[5] = 0; /*   */
    baudsetup[6] = 8; /* 8     */

	/* delay >10ms for USB reset */
    __delay_cycles(150000);
}

/*****************************************************************************

******************************************************************************/
__C_task main(void)
{	
	hardwareInit();
  	AVR910_Init();
    usbInit();
    SEI();
	
    // the host eats the first packet -- don't know why...
    usbSetInterrupt(0, 0);

    // main event loop
    for(;;)
    {
        UCHAR *data;
        UCHAR bytesRead;
        usbPoll();

        if(usbAllRequestsAreDisabled())
        	usbEnableAllRequests();
        		
        //
        if(usbInterruptIsReady() && (bytesRead = AVR910_IsRxData(&data)) > 0)
        {
            //       USB
          	if(bytesRead > 7)
            	if(bytesRead == 8)
                	bytesRead = 7;
                else
                	bytesRead = 8;

            //      USB    
            //    
            usbSetInterrupt(data, bytesRead);
			AVR910_ClearTXData(bytesRead);
        }

        // We need to report rx and tx carrier after open attempt
        if(intr3Status != 0 && usbInterruptIsReady3())
        {	
          	static UCHAR serialStateNotification[10] = {0xa1, 0x20, 0, 0, 0, 0, 2, 0, 3, 0};
            if(intr3Status == 2)
            	usbSetInterrupt3(serialStateNotification, 8);
            else
            	usbSetInterrupt3(serialStateNotification+8, 2);
            intr3Status--;
        }
    }
}

